#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from zope.container.interfaces import IContainer, IContained
from zope.location.interfaces import ISublocations

# import packages
from BTrees.OOBTree import OOBTree
from persistent.list import PersistentList
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyramid.threadlocal import get_current_registry
from zope.container.ordered import OrderedContainer


class BTreeOrderedContainer(OrderedContainer):
    """BTree based ordered container"""

    def __init__(self):
        self._data = OOBTree()
        self._order = PersistentList()


@adapter_config(context=IContained, provides=ISublocations)
class ContainerSublocationsAdapter(ContextAdapter):
    """Container sub-locations adapter"""

    def sublocations(self):
        context = self.context
        # Check for adapted sub-locations first...
        registry = get_current_registry()
        for name, adapter in registry.getAdapters((context,), ISublocations):
            if not name:  # don't reuse default adapter!!
                continue
            for location in adapter.sublocations():
                yield location
        # then yield container items
        if IContainer.providedBy(context):
            for key in context:
                yield context[key]


def find_objects_matching(root, condition, ignore_root=False):
    """Find all objects in root that match the condition

    The condition is a Python callable object that takes an object as
    argument and must return a boolean result.

    All sub-objects of the root will also be searched recursively.
    """
    if (not ignore_root) and condition(root):
        yield root
    locations = ISublocations(root, None)
    if locations is not None:
        for location in locations.sublocations():
            if condition(location):
                yield location
            for sublocation in find_objects_matching(location, condition, ignore_root=True):
                yield sublocation


def find_objects_providing(root, interface):
    """Find all objects in root that provide the specified interface

    All sub-objects of the root will also be searched recursively.
    """
    for match in find_objects_matching(root, interface.providedBy):
        yield match
